@@ -4,7 +4,7 @@ use warnings;
 use base qw/Exporter/;
 use 5.008001;
 
-our $VERSION = '2.02';
+our $VERSION = '2.03';
 
 use Carp ();
 use Furl::ConnectionCache;
@@ -173,11 +173,17 @@ sub _parse_url {
     $url =~ m{\A
         ([a-z]+)                    # scheme
         ://
+        (?:
+            ([^/:@?]+) # user
+            :
+            ([^/:@?]+) # password
+            @
+        )?
         ([^/:?]+)                   # host
         (?: : (\d+) )?              # port
         (?: ( /? \? .* | / .*)  )?  # path_query
     \z}xms or Carp::croak("Passed malformed URL: $url");
-    return( $1, $2, $3, $4 );
+    return( $1, $2, $3, $4, $5, $6 );
 }
 
 sub make_x_www_form_urlencoded {
@@ -211,9 +217,9 @@ sub request {
 
     my $timeout_at = time + $self->{timeout};
 
-    my ($scheme, $host, $port, $path_query);
+    my ($scheme, $username, $password, $host, $port, $path_query);
     if (defined(my $url = $args{url})) {
-        ($scheme, $host, $port, $path_query) = $self->_parse_url($url);
+        ($scheme, $username, $password, $host, $port, $path_query) = $self->_parse_url($url);
     }
     else {
         ($scheme, $host, $port, $path_query) = @args{qw/scheme host port path_query/};
@@ -263,14 +269,23 @@ sub request {
     if(!$in_keepalive) {
         my $err_reason;
         if ($proxy) {
-            my (undef, $proxy_host, $proxy_port, undef)
+            my (undef, $proxy_user, $proxy_pass, $proxy_host, $proxy_port, undef)
                 = $self->_parse_url($proxy);
+            my $proxy_authorization;
+            if (defined $proxy_user) {
+                _requires('MIME/Base64.pm',
+                    'Basic auth');
+                $proxy_authorization = 'Basic ' . MIME::Base64::encode_base64("$proxy_user:$proxy_pass");
+            }
             if ($scheme eq 'http') {
                 ($sock, $err_reason)
                     = $self->connect($proxy_host, $proxy_port, $timeout_at);
+                if (defined $proxy_authorization) {
+                    $self->{proxy_authorization} = $proxy_authorization;
+                }
             } else {
                 ($sock, $err_reason) = $self->connect_ssl_over_proxy(
-                    $proxy_host, $proxy_port, $host, $port, $timeout_at);
+                    $proxy_host, $proxy_port, $host, $port, $timeout_at, $proxy_authorization);
             }
         } else {
             if ($scheme eq 'http') {
@@ -316,6 +331,13 @@ sub request {
             }
         }
         unshift @headers, 'Connection', $connection_header;
+        if (exists $self->{proxy_authorization}) {
+            push @headers, 'Proxy-Authorization', $self->{proxy_authorization};
+        }
+        if (defined $username) {
+            _requires('MIME/Base64.pm', 'Basic auth');
+            push @headers, 'Authorization', 'Basic ' . MIME::Base64::encode_base64("${username}:${password}");
+        }
 
         my $content       = $args{content};
         my $content_is_fh = 0;
@@ -607,12 +629,16 @@ sub connect_ssl {
 }
 
 sub connect_ssl_over_proxy {
-    my ($self, $proxy_host, $proxy_port, $host, $port, $timeout_at) = @_;
+    my ($self, $proxy_host, $proxy_port, $host, $port, $timeout_at, $proxy_authorization) = @_;
     _requires('IO/Socket/SSL.pm', 'SSL');
 
     my $sock = $self->connect($proxy_host, $proxy_port, $timeout_at);
 
-    my $p = "CONNECT $host:$port HTTP/1.0\015\012Server: $host\015\012\015\012";
+    my $p = "CONNECT $host:$port HTTP/1.0\015\012Server: $host\015\012";
+    if (defined $proxy_authorization) {
+        $p .= "Proxy-Authorization: $proxy_authorization\015\012";
+    }
+    $p .= "\015\012";
     $self->write_all($sock, $p, $timeout_at)
         or return $self->_r500(
             "Failed to send HTTP request to proxy: " . _strerror_or_timeout());
